@@ -1,8 +1,9 @@
 import time
 import urllib
-from flask import render_template, request, g
+from flask import render_template, request, g, jsonify
+from flask.ext.login import current_user
 from wikked.views import (get_page_meta, get_page_or_404, get_page_or_none,
-        is_page_readable, make_auth_response,
+        is_page_readable,
         url_from_viewarg, split_url_from_viewarg,
         CHECK_FOR_READ)
 from wikked.web import app
@@ -12,7 +13,7 @@ from wikked.scm.base import STATE_NAMES
 @app.route('/')
 def home():
     tpl_name = 'index.html'
-    if app.config['DEBUG']:
+    if app.config['DEV_ASSETS']:
         tpl_name = 'index-dev.html'
     return render_template(tpl_name, cache_bust=('?%d' % time.time()));
 
@@ -20,7 +21,7 @@ def home():
 @app.route('/read/<path:url>')
 def read():
     tpl_name = 'index.html'
-    if app.config['DEBUG']:
+    if app.config['DEV_ASSETS']:
         tpl_name = 'index-dev.html'
     return render_template(tpl_name, cache_bust=('?%d' % time.time()));
 
@@ -28,7 +29,7 @@ def read():
 @app.route('/search')
 def search():
     tpl_name = 'index.html'
-    if app.config['DEBUG']:
+    if app.config['DEV_ASSETS']:
         tpl_name = 'index-dev.html'
     return render_template(tpl_name, cache_bust=('?%d' % time.time()));
 
@@ -43,7 +44,7 @@ def api_list_pages(url):
     pages = filter(is_page_readable, g.wiki.getPages(url_from_viewarg(url)))
     page_metas = [get_page_meta(page) for page in pages]
     result = {'path': url, 'pages': list(page_metas)}
-    return make_auth_response(result)
+    return jsonify(result)
 
 
 @app.route('/api/read/')
@@ -53,7 +54,19 @@ def api_read_main_page():
 
 @app.route('/api/read/<path:url>')
 def api_read_page(url):
-    #TODO: remove redundant quoting/spliting/unquoting around here.
+    additional_info = {}
+    if 'user' in request.args:
+        user = current_user
+        if user.is_authenticated():
+            additional_info['user'] = {
+                    'username': user.username,
+                    'groups': user.groups
+                    }
+        else:
+            additional_info['user'] = False
+
+    force_resolve = ('force_resolve' in request.args)
+
     endpoint, value, path = split_url_from_viewarg(url)
     if endpoint is None:
         # Normal page.
@@ -61,18 +74,37 @@ def api_read_page(url):
                 path,
                 convert_url=False,
                 check_perms=CHECK_FOR_READ,
-                force_resolve=('force_resolve' in request.args))
+                force_resolve=force_resolve)
 
         result = {'meta': get_page_meta(page), 'text': page.text}
-        return make_auth_response(result)
+        result.update(additional_info)
+        return jsonify(result)
 
-    # Meta listing page.
+    # Meta listing page or special endpoint.
     meta_page_url = '%s:%s' % (endpoint, path)
     info_page = get_page_or_none(
             meta_page_url,
             convert_url=False,
             check_perms=CHECK_FOR_READ,
-            force_resolve=('force_resolve' in request.args))
+            force_resolve=force_resolve)
+
+    endpoint_info = g.wiki.endpoints.get(endpoint)
+    if endpoint_info is not None:
+        # We have some information about this endpoint...
+        if endpoint_info.default and info_page is None:
+            # Default page text.
+            info_page = get_page_or_404(
+                    endpoint_info.default,
+                    convert_url=False,
+                    check_perms=CHECK_FOR_READ,
+                    force_resolve=force_resolve)
+
+        if not endpoint_info.query:
+            # Not a query-based endpoint (like categories). Let's just
+            # return the text.
+            result = {'meta': get_page_meta(info_page), 'text': info_page.text}
+            result.update(additional_info)
+            return jsonify(result)
 
     # Get the list of pages to show here.
     query = {endpoint: [value]}
@@ -103,7 +135,8 @@ def api_read_page(url):
     if info_page:
         result['meta'] = get_page_meta(info_page)
 
-    return make_auth_response(result)
+    result.update(additional_info)
+    return jsonify(result)
 
 
 @app.route('/api/raw/')
@@ -115,7 +148,7 @@ def api_read_main_page_raw():
 def api_read_page_raw(url):
     page = get_page_or_404(url, check_perms=CHECK_FOR_READ)
     result = {'meta': get_page_meta(page), 'text': page.raw_text}
-    return make_auth_response(result)
+    return jsonify(result)
 
 
 @app.route('/api/query')
@@ -126,7 +159,7 @@ def api_query():
             'query': query,
             'pages': [get_page_meta(p) for p in pages]
         }
-    return make_auth_response(result)
+    return jsonify(result)
 
 
 @app.route('/api/state/')
@@ -138,7 +171,7 @@ def api_get_main_page_state():
 def api_get_state(url):
     page = get_page_or_404(url, check_perms=CHECK_FOR_READ)
     state = page.getState()
-    return make_auth_response({
+    return jsonify({
         'meta': get_page_meta(page, True),
         'state': STATE_NAMES[state]
         })
@@ -154,8 +187,8 @@ def api_get_outgoing_links(url):
     page = get_page_or_404(url, check_perms=CHECK_FOR_READ)
     links = []
     for link in page.links:
-        other = get_page_or_none(link)
-        if other is not None:
+        other = get_page_or_none(link, convert_url=False)
+        if other is not None and is_page_readable(other):
             links.append({
                 'url': other.url,
                 'title': other.title
@@ -164,7 +197,7 @@ def api_get_outgoing_links(url):
             links.append({'url': link, 'missing': True})
 
     result = {'meta': get_page_meta(page), 'out_links': links}
-    return make_auth_response(result)
+    return jsonify(result)
 
 
 @app.route('/api/inlinks/')
@@ -177,7 +210,7 @@ def api_get_incoming_links(url):
     page = get_page_or_404(url, check_perms=CHECK_FOR_READ)
     links = []
     for link in page.getIncomingLinks():
-        other = get_page_or_none(link)
+        other = get_page_or_none(link, convert_url=False)
         if other is not None and is_page_readable(other):
             links.append({
                 'url': link,
@@ -187,5 +220,5 @@ def api_get_incoming_links(url):
             links.append({'url': link, 'missing': True})
 
     result = {'meta': get_page_meta(page), 'in_links': links}
-    return make_auth_response(result)
+    return jsonify(result)
 
